

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

   File:           vissweg.h                                         
                                                                        
   Classification: UNCLASSIFIED                                   
                                                                        
   Description:    This file contains the macros, structures,
                   and classes needed access SWEG simulation data
                   for visualization.

   Written by:     Alex Harper
*********************************************************************** */


#if !defined __VISSWEG_H
#define __VISSWEG_H

// ***********************************************************

#include "vissim.h"
#include "parse_tdb.h"
#include "prototypes.h"

// ***********************************************************

#define USE_SCRAMNET 0
#define USE_SHMEM    1

#define SWEG_ASSET_READY    2
#define SWEG_START_SCENARIO 4
#define SWEG_PAUSE          5

#define AAM_LETHAL_ENGAGE                   143300
#define AAM_SYSTEM_DEATH                    226000
#define AAM_SENSOR_TRACKING_CHANGE          261100
#define AAM_REQUEST_SENSOR_TRACK_CHANGE     266100
#define AAM_MODE_CHANGE                     267100
#define AAM_FREQUENCY_CHANGE                267200
#define AAM_EMITTER_SYSTEM_OPERATION_STATUS 267400
#define AAM_FUSED_PERCEPTION                332101
#define AAM_DECISION_TO_FIRE                333430
#define AAM_WEAPON_FIRE_DECISION            333431
#define AAM_GENERAL_BIRTH_ANNOUNCMENT       411000
#define AAM_GENERAL_BIRTH_ANNOUNCMENT1      411010
#define AAM_CORRELATED_BIRTH_ANNOUNCEMENT   411100
#define AAM_REQUEST_CREATE_PLAYER           411210
#define AAM_REQUEST_CREATED_PLAYER          411220
#define AAM_PLATFORM_LEAVES_EXERCISE        411310
#define AAM_END_SCENARIO                    499999

// incidents from SWEG
#define AAM_FIRES_A_WEAPON_AT               511020
#define AAM_SUCCESSFULLY_HIT                511023
#define AAM_UNFORTUNATELY_MISSED            511031
#define AAM_LEAVES_THE_SCENARIO             511075

// incidents to SWEG
#define AAM_EXTERN_FIRES_A_WEAPON_AT        512020
#define AAM_EXTERN_SUCCESSFULLY_HIT         512023
#define AAM_EXTERN_UNFORTUNATELY_MISSED     512031


#define DEAD_PLATFORM     10000
#define DEAD_SYSTEM          10

#define SYSTEM_TYPE     1000000
#define SYSTEM_SUBTYPE   100000

#define SYSTEM_TYPE_SENSOR   2

#define SYSTEM_SUBTYPE_RECEIVER    1
#define SYSTEM_SUBTYPE_TRANSMITTER 2

#define INTERFACE_FLAG_OVERRIDE 0x010000
#define INTERFACE_FLAG_READY    0x020000

//#define PERFORM_QUERY_SYMBOL_CODE     0x00010000
#define PERFORM_QUERY_MAX_RADAR_RANGE 0x00020000
#define PERFORM_ADD_PLATFORM          0x00030000

#define SENSOR_MODE_SEARCHING 1
#define SENSOR_MODE_ACQUIRE   2
#define SENSOR_MODE_TRACKING  3
#define SENSOR_MODE_GUIDANCE  4


/* **************************************************************
************************************************************** */
class sweg_sensor_type : public dbl_llist {

   public:
      System *sensor_rcvr;
      int semcode_rcvr;       // semantic code
      System *sensor_xmit;
      int semcode_xmit;       // semantic code
      int id;
      radar_type *sensor_data;
      symbol_correlation_type *str;
      int mode;

      sweg_sensor_type() { str = NULL; sensor_rcvr = sensor_xmit = NULL; sensor_data = NULL; id = 0; mode = SENSOR_MODE_SEARCHING; }
      virtual ~sweg_sensor_type() {}
};


/* **************************************************************
************************************************************** */
class sweg_weapon_type : public dbl_llist {

   public:
      System *weapon;
      int semcode;       // semantic correlation code

      virtual ~sweg_weapon_type() {}
};


/* **************************************************************
   This class is a subclass of the basic 3D graphics manager for any
   SWEG related object to be displayed
************************************************************** */
class sweg_platform : public basic_platform {

   protected:
      // basic_platform
      void update_data(double *machine_time);

      // sweg_platform
      int build_sensor_list(iface *isim);
      int update_sensors(iface *isim);
      
   public:
      // basic_platform
      int  perform(iface *isim, int task, void *data);
      void update_platform(plat_stat_type *pc);

      // sweg_platform
      Platform *plat;       // sweg platform in SWEDAT
      double birth_time;
      dbl_llist_manager weaponman;
      dbl_llist_manager sensorman;

      sweg_platform()  { plat = NULL; birth_time = -1; }
      virtual ~sweg_platform() {};
      
      int init(iface *isim);
};


/* **************************************************************
  This class is a subclass of the basic simulation manager for
  interfacing w/ a SWEG scenario
************************************************************** */
class sweg_iface : public iface {

   protected:
   
      // sweg_iface
      sweg_platform control_platform;
      Asset_mailbox *rec_mail;    // sweg mailbox
      Asset_mailbox *send_mail;   // mailbox for messages from sweg master
      double deathcheck;
      
      basic_platform *CreatePlatformList(double *machine_time);      
      basic_platform *AddPlatform(Platform *plat, double *sim_time, double *machine_time);
      void BurnDriftwood(double *machine_time);

      virtual int process_messages(Asset_action_message *mess, double *machine_time);

   public:

      // iface
      void label(dbl_llist_manager *label_manager);
      void parse_input(int argc, char **argv);
      void init();
      void update(double *machine_time);
      void mcp(plat_stat_type *pc);
      int  get_data(int type, void *data);
      int  set_data(int type, void *data);
      
      // sweg_iface
      int SWEDAT_LOCATION; // flag for local or scramnet
      int ASSET_TYPE;      // asset id
      int USER_ID;         // user id
      TDB tdb_parser;
      string_type tdbdatname, tdblisname;

      sweg_iface();
      virtual ~sweg_iface() {}

      int find_template(int type, Asset_action_message **aam);

      Semantic_Correlation *find_correlation(int code);
      Semantic_Correlation *find_user_defined_correlation(int code);

      void *QueryControlData() { return &control_platform; }
      int get_messages(double *machine_time);
      int find_messages(int type, int *last_index, Asset_action_message **mess);
};

#endif
